/*
*  Name:      base_data_source.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Definition of a general query class
*  Date:      $Date: 2002/04/28 22:56:19 $
*  Revision:  $Revision: 1.2 $
*
*  Copyright (C) 2001  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/
#ifndef _UESQLC_BASE_DATA_SOURCE__
#define _UESQLC_BASE_DATA_SOURCE__

#include <cstdlib>
#include <mpcl/string.hh>


///  Name-space for UESQL.
namespace uesql
{

  /**
  *  Base class for data sources.
  *  @see TQuery.
  */
  template <typename TInstance, typename TSize = std::size_t>
  class TBaseDataSource
  {

    public:

      ///  Size type.
      typedef
        TSize
        size_type;


    public:

      //
      //  C O N S T R U C T O R S
      //

      /**
      *  Closes the cursor.
      *  @param rkyCURSOR_NAME Cursor name.
      */
      virtual void closeCursor (const mpcl::TString& rkyCURSOR_NAME) = 0;

      /**
      *  Declares the cursor, and returns the tuple count of the query.
      *  @param rkyCURSOR_NAME Cursor name.
      *  @return Tuple count of the query.
      */
      virtual size_type declareCursor (const mpcl::TString& rkyCURSOR_NAME) = 0;

      /**
      *  Gets  the data from  next tuple in the  cursor named \a rkyCURSOR_NAME
      *  and puts it into \a rtINSTANCE.
      *  @param rkyCURSOR_NAME Cursor name.
      *  @param rtINSTANCE     Data holder instance.
      */
      virtual void fetchNext ( const mpcl::TString& rkyCURSOR_NAME ,
                               TInstance&           rtINSTANCE     ) = 0;

      /**
      *  Gets the  data from  tuple at  position \a zINDEX in the cursor  named
      *  \a rkyCURSOR_NAME and puts it into \a rtINSTANCE.
      *  @param rkyCURSOR_NAME Cursor name.
      *  @param rtINSTANCE     Data holder instance.
      *  @param zINDEX         Position of the tuple in the query.
      */
      virtual void fetchAbsolute ( const mpcl::TString& rkyCURSOR_NAME ,
                                   TInstance&           rtINSTANCE     ,
                                   size_type            zINDEX         ) = 0;

      /**
      *  Opens the cursor.
      *  @param rkyCURSOR_NAME Cursor name.
      */
      virtual void openCursor (const mpcl::TString& rkyCURSOR_NAME) = 0;

      ///  Activates error exception.
      virtual void setErrorException (void) = 0;

  };  // class TBaseDataSource

}  // namespace uesql


#endif  // not _UESQLC_BASE_DATA_SOURCE__
