#include <mpcl/log.hh>
#include "postgresql-libpq-6.2.x.hh"
/*
*  Name:         create-iterators.uesql.cc
*  Author:       Francisco Vides Fernandez
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Database creation of iterators test
*  Date:         $Date: 2002/08/19 12:59:57 $
*  Revision:     $Revision: 1.3 $
*
*  Copyright (C) 2001-2002  Francisco Vides Fernandez <pax@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/general-exceptions.hh>
#include <mpcl/log.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>

using uesql::TExceptionUESQL_Sqlerror;


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;


//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::TString _GetErrorMessage (void)
{

  using mpcl::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesql::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static void _CreateTestSchema ( const mpcl::TString& rkyDBMS_SERVER_NAME ,
                                const mpcl::TString& rkyUSER_NAME        ,
                                const mpcl::TString& rkyUSER_PASSWORD    )
{

  using uesql::ComposeSchemaLocation;
  using uesql::ComposeUserLogin;
  using uesql::TString;

  TString   yDbmsSchemaLocation;
  TString   yDbmsTestSchemaLocation;
  TString   yDbmsUserLogin;

  yDbmsSchemaLocation     = ComposeSchemaLocation (rkyDBMS_SERVER_NAME, rkyUSER_NAME);
  yDbmsTestSchemaLocation = ComposeSchemaLocation (rkyDBMS_SERVER_NAME, "TEST_ITERATORS_SCHEMA");
  yDbmsUserLogin          = ComposeUserLogin (rkyUSER_NAME, rkyUSER_PASSWORD);
  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION      = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN           = %s", yDbmsUserLogin.c_str());
  TEST_WRITE_INFO ("DBMS-TEST-SCHEMA-LOCATION = %s", yDbmsTestSchemaLocation.c_str());
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
{
mpcl::TString   hySqlServerDescriptor;
mpcl::TString   hySqlServerName;
mpcl::TString   hyDatabaseName;
mpcl::TString   hyConnectionName;
mpcl::TString   hyUserLogin;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesql::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesql::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = "DEFAULT";
  hyUserLogin           = uesql::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
  if ( uesql::htPostgresql62xManager.isConnected (hyConnectionName) )
  {
    uesql::htPostgresql62xManager.unbind (hyConnectionName);
  }
}
if ( uesql::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesql::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
  {
    
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
  }
  else
  {
    uesql::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
{
PGresult*       ptPgresult;
mpcl::TString   hyConnectionName;
mpcl::TString   hyLastConnectionName = uesql::htPostgresql62xManager.currentConnectionName();

uesql::htPostgresql62xManager.clearQueues();

hyConnectionName = uesql::htPostgresql62xManager.instantiate ("TEST_ITERATORS_SCHEMA");
hyConnectionName.lowercase();
ptPgresult = uesql::htPostgresql62xManager.execute (mpcl::Format ("CREATE DATABASE %s;", hyConnectionName.c_str()));
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
}
uesql::htPostgresql62xManager.connect (hyConnectionName, hyConnectionName);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
}
else
{
  uesql::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  
{
PGresult*   ptPgresult;

ptPgresult = uesql::htPostgresql62xManager.execute ("CREATE TABLE INSTANCE (ATTRIB0 DATE, ATTRIB1 INT4, ATTRIB2 FLOAT8, ATTRIB3 VARCHAR (20));");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
}
}
  uesql::htPostgresql62xManager.unbindCurrent();
  uesql::htPostgresql62xManager.setCurrentConnection (hyLastConnectionName);
}
}
{
mpcl::TString   hySqlServerDescriptor;
mpcl::TString   hySqlServerName;
mpcl::TString   hyDatabaseName;
mpcl::TString   hyConnectionName;
mpcl::TString   hyUserLogin;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (yDbmsTestSchemaLocation);
uesql::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesql::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = uesql::htPostgresql62xManager.instantiate ("_'test_connection'");
  hyUserLogin           = uesql::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hyConnectionName.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
}
if ( uesql::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesql::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
  {
    
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
  }
  else
  {
    uesql::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
}  // not PostgreSQL LIBPQ 6.2.x

}  // _CreateTestSchema()


static void _FillSchema (void)
{

{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO INSTANCE VALUES ('20-12-2000', 0, 0.0, 'valor 0');");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
}
}
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO INSTANCE VALUES ('20-12-2001', 1, 1.1, 'valor 1');");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
}
}
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO INSTANCE VALUES ('20-12-2002', 2, 2.2, 'valor 2');");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  
throw uesql::TExceptionUESQL_Sqlerror (__FILE__, __LINE__);
}
}
}  // not PostgreSQL LIBPQ 6.2.x

}   // _FillSchema()


int main (int argc, const char** argv)
{

  mpcl::TConfigProcessor   tConfigProcessor (argc, argv);

  TEST_INIT ("Database creation for iterators test");
  TEST_START_WATCH;
  try
  {
    tConfigProcessor.addOption ("dbms-server-name",   "o", "dbms");
    tConfigProcessor.addOption ("dbms-user-name",     "u", "test");
    tConfigProcessor.addOption ("dbms-user-password", "p", "test");
    tConfigProcessor["dbms-server-name"].setUsageString ("Server where database resides");
    tConfigProcessor["dbms-user-name"].setUsageString ("");
    tConfigProcessor["dbms-user-password"].setUsageString ("");
    tConfigProcessor.processOptions();
    
    if ( !TEST_IN_SILENT_MODE )
    {
      mpcl::tLog.also (std::cerr);
    }
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//

}  // not PostgreSQL LIBPQ 6.2.x
    _CreateTestSchema ( tConfigProcessor ["dbms-server-name"].getValue()   ,
                        tConfigProcessor ["dbms-user-name"].getValue()     ,
                        tConfigProcessor ["dbms-user-password"].getValue() );
    _FillSchema();
  }
  catch (const uesql::TExceptionUESQL_Sqlerror& rktEXCEPTION)
  {
    std::cerr << "SQLSTATE="    << uesql::SQLSTATE    << std::endl;
    std::cerr << "SQLCODE="     << uesql::SQLCODE     << std::endl;
    std::cerr << "DESCRIPTION=" << _GetErrorMessage() << std::endl;
    TEST_FAIL_WITH_CODE (uesql::SQLCODE);
  }
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (...)
  {
    std::cerr << "Unhandled exception" << std::endl;
    TEST_FAIL_WITH_CODE (3);
  }
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.erase (std::cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
